# requires the library https://pypi.org/project/python-chess/

import chess, chess.pgn, sys, os.path, collections

GAME_TYPES = 7
THRESHOLD_EASY = 3
THRESHOLD_HARD = 7
GAME_TYPE_PROMOTED = "promoted"
# chess.PAWN = 1
# chess.KNIGHT = 2
# chess.BISHOP = 3
# chess.ROOK = 4
# chess.QUEEN = 5
# chess.KING = 6

is_easy = sys.argv[1] == "easy"
threshold = THRESHOLD_EASY if is_easy else THRESHOLD_HARD
pgn_files = [os.path.join(os.path.dirname(os.path.abspath(__file__)), "optimal.pgn"), sys.argv[2]]

class DumbVisitor(chess.pgn.GameModelCreator):
    def visit_header(self, tagname, tagvalue):
        if tagname == 'Variant' and tagvalue != 'chess':
            print("Wrong answer: Game %d has 'Variant' tag, whose value is not 'chess'." % game_id)
            sys.exit(1)

        if tagname == 'FEN' and tagvalue != chess.STARTING_FEN:
            print("Wrong answer: Game %d has 'FEN' tag and its value is incorrect" % game_id)
            sys.exit(1)

    def visit_nag(self, nag):
        pass

    def begin_variation(self):
        pass

    def end_variation(self):
        pass

    def visit_result(self, result):
        pass

    def visit_comment(self, comment):
        pass

    def handle_error(self, error):
        print("Wrong answer: Unable to parse game %d: %s" % (game_id, str(error)))
        sys.exit(1)

# solutions[0] will be our solution, solutions[1] contestant's
solutions = []
for pgn_file in pgn_files:
    with open(pgn_file) as pgn:
        game_id = 1
        solution = collections.defaultdict(lambda: 100)
        while True:
            try:
                game = chess.pgn.read_game(pgn, DumbVisitor)
            except:
                print("Wrong answer: failed to parse game %d (use 7-bit ASCII only)" % game_id)
                sys.exit(1)
            if game is None:
                break
            board = game.board()

            uci = []
            for move in game.main_line():
                uci.append(board.uci(move))
                board.push(move)

            mating_piece = board.piece_at(move.to_square)

            if board.turn == chess.WHITE:
                print("Wrong answer: Game %d does not end with white's move" % game_id)
                sys.exit(1)

            if not board.is_checkmate():
                print("Wrong answer: Game %d did not end with checkmate" % game_id)
                sys.exit(1)

            for i,(w,b) in enumerate(zip(uci[:-1:2], uci[1::2])):
                if len(w) != len(b):
                    print("Wrong answer: game %d move %d is not a mirror: %s %s" % (game_id, i+1, w, b))
                    sys.exit(1)
                for c,d in zip(w,b):
                    if c != d and ord(c) + ord(d) != 2*ord('0') + 9:
                        print("Wrong answer: game %d move %d is not a mirror: %s %s" % (game_id, i+1, w, b))
                        sys.exit(1)

            if board.promoted & (1<<move.to_square):
                solution[GAME_TYPE_PROMOTED] = min(solution[GAME_TYPE_PROMOTED], board.fullmove_number)
            else:
                solution[mating_piece] = min(solution[mating_piece], board.fullmove_number)
            game_id += 1

        solutions.append(solution)

better,equal,worse=0,0,0
for k,v in solutions[0].items():
    if solutions[1][k] < v:
        better += 1
    elif solutions[1][k] == v:
        equal += 1
    else:
        worse += 1

# if better: print("Better solution(s): %s" % str(solutions[1]))

if equal + better >= threshold:
    print("OK")
else:
    print("Wrong answer: Needed {} points, only found {} better and {} equally good solutions as we did.".format(threshold,better,equal))
    sys.exit(1)
